﻿namespace Model;

public interface IDataManager
{
    // Create
    void AddAlbum(Album album);

    void AddAlbums(List<Album> albumsList);

    void AddArtist(Artist artist);

    void AddArtists(List<Artist> artistsList);

    void AddPlaylist(Playlist playlist);

    void AddPlaylists(List<Playlist> playlistsList);

    void AddCustomTitle(CustomTitle title);

    void AddCustomTitles(List<CustomTitle> customTitlesList);

    void AddInfoTitle(InfoTitle title);

    void AddInfoTitles(List<InfoTitle> infoTitlesList);

    // Read
    List<CustomTitle> GetCustomTitles();

    CustomTitle? GetCustomTitleByUrl(string custom);

    List<InfoTitle> GetInfoTitles();

    InfoTitle? GetInfoTitleByUrl(string url);

    List<Album> GetAlbums();

    Album? GetAlbumByUrl(string url);

    List<Artist> GetArtists();

    Artist? GetArtistByName(string name);

    List<Playlist> GetPlaylists();

    Playlist? GetPlaylistByUrl(string url);


    // Update
    void UpdateCustomTitle(CustomTitle title, string name, string url, string info, string path);

    void UpdateCustomTitleByUrl(string url, string name, string newUrl, string info, string path);

    void UpdateInfoTitle(InfoTitle title, string name, string url, string info, Artist artist, string description, Genre genre);

    void UpdateInfoTitleByName(string url, string name, string newUrl, string info, Artist artist, string description, Genre genre);

    void UpdateInfoTitleByArtistName(InfoTitle title, string name, string url, string info, string artist, string description, Genre genre);

    void UpdateInfoTitleByNameByArtistName(string url, string name, string newUrl, string info, string artist, string description, Genre genre);

    void UpdateAlbum(Album album, string name, string url, Artist artist, string description, string info);

    void UpdateAlbumByUrl(string url, string name, string newUrl, Artist artist, string description, string info);

    void UpdateAlbumByArtistName(Album album, string name, string url, string artist, string description, string info);

    void UpdateAlbumByUrlByArtistName(string url, string name, string newUrl, string artist, string description, string info);

    void UpdatePlaylist(Playlist playlist, string name, string description, string url);

    void UpdatePlaylistByUrl(string url, string name, string description, string newUrl);

    void UpdateArtist(Artist artist, string name);

    void UpdateArtistByName(string name, string newName);

    // Delete
    void RemoveAlbum(Album album);

    void RemoveAlbums(List<Album> albumsList);

    void RemoveArtist(Artist artist);

    void RemoveArtists(List<Artist> artistsList);

    void RemovePlaylist(Playlist playlist);

    void RemovePlaylists(List<Playlist> playlistsList);

    void RemoveCustomTitle(CustomTitle title);

    void RemoveCustomTitles(List<CustomTitle> customTitlesList);

    void RemoveInfoTitle(InfoTitle title);

    void RemoveInfoTitles(List<InfoTitle> infoTitlesList);

    // Serialization
    void LoadSerialization();

    void SaveSerialization();

    // Exists
    bool ExistsPlaylist(Playlist playlist);

    bool ExistsPlaylistByUrl(string url);

    bool ExistsAlbum(Album album);

    bool ExistsAlbumByUrl(string url);

    bool ExistsArtist(Artist artist);

    bool ExistsArtistByName(string name);

    bool ExistsCustomTitle(CustomTitle title);

    bool ExistsCustomTitleByUrl(string url);

    bool ExistsInfoTitle(InfoTitle title);

    bool ExistsInfoTitleByUrl(string url);
}